@@ -18,6 +18,7 @@ from typing import (
 )
 
 from apischema.conversions.conversions import (
+    Conversion,
     Conversions,
     ResolvedConversion,
     ResolvedConversions,
@@ -145,29 +146,28 @@ class DeserializationVisitor(ConversionsVisitor[Deserialization, Return]):
         tp: Type, conversions: ResolvedConversions
     ) -> Union[Deserialization, None, UndefinedType]:
         origin = get_origin_or_type(tp)
-        result = [
-            conv
-            for conv in conversions
-            if issubclass(get_origin_or_type(conv.target), origin)
-        ]
-        for i, conv in enumerate(result):
+        identity_conv = False
+        result = []
+        for conv in conversions:
             if is_identity(conv):
-                if len(result) == 1:
-                    return Undefined
-                else:
-                    namespace = {
-                        "__new__": lambda _, *args, **kwargs: origin(*args, **kwargs),
-                        SELF_CONVERSION_ATTR: True,
-                    }
-                    wrapper = new_class(
-                        f"{origin.__name__}SelfDeserializer",
-                        (tp,),
-                        exec_body=lambda ns: ns.update(namespace),
-                    )
-                    result[i] = ResolvedConversion(
+                identity_conv = True
+                namespace = {
+                    "__new__": lambda _, *args, **kwargs: origin(*args, **kwargs),
+                    SELF_CONVERSION_ATTR: True,
+                }
+                wrapper = new_class(
+                    f"{origin.__name__}SelfDeserializer",
+                    (tp,),
+                    exec_body=lambda ns: ns.update(namespace),
+                )
+                result.append(
+                    ResolvedConversion(
                         replace(conv, source=wrapper, target=conv.target)
                     )
-        return result or None
+                )
+            elif issubclass(get_origin_or_type(conv.target), origin):
+                result.append(conv)
+        return Undefined if identity_conv and len(result) == 1 else result or None
 
     _default_conversions = staticmethod(_deserializers.get)  # type: ignore
 
@@ -237,8 +237,10 @@ class SerializationVisitor(ConversionsVisitor[Serialization, Return]):
     ) -> Union[Serialization, None, UndefinedType]:
         origin = get_origin_or_type(tp)
         for conv in conversions:
-            if issubclass(origin, get_origin_or_type(conv.source)):
-                return Undefined if is_identity(conv) else conv
+            if is_identity(conv):
+                return Undefined
+            elif issubclass(origin, get_origin_or_type(conv.source)):
+                return conv
         else:
             return None
 
@@ -246,7 +248,14 @@ class SerializationVisitor(ConversionsVisitor[Serialization, Return]):
     def _default_conversions(tp: Type) -> Optional[Conversions]:
         for sub_cls in tp.__mro__:
             if sub_cls in _serializers:
-                return _serializers[sub_cls]
+                conversion = _serializers[sub_cls]
+                if (
+                    sub_cls == tp
+                    or not isinstance(conversion, Conversion)
+                    or conversion.inherited is None
+                    or conversion.inherited
+                ):
+                    return conversion
         else:
             return None
 
