@@ -720,9 +720,11 @@ class DeserializationMethodVisitor(
         return factory
 
     def typed_dict(
-        self, cls: Type, keys: Mapping[str, AnyType], total: bool
+        self, cls: Type, keys: Mapping[str, AnyType], required_keys: Collection[str]
     ) -> DeserializationMethodFactory:
+        # TypedDict doesn't use aliaser, because it cannot be used in serialization
         items_deserializers = {key: self.method(tp) for key, tp in keys.items()}
+        required_keys = set(required_keys)
 
         @DeserializationMethodFactory.from_type(cls)
         def factory(
@@ -732,21 +734,17 @@ class DeserializationMethodVisitor(
             def method(ctx: DeserializationContext, data: Any) -> Any:
                 data = ctx.coercer(dict, data)
                 items: Dict[str, Any] = {}
-                key_count = 0
                 item_errors: Dict[ErrorKey, ValidationError] = {}
                 for key, value in data.items():
                     if key in items_deserializers:
-                        key_count += 1
                         try:
                             items[key] = items_deserializers[key](ctx, value)
                         except ValidationError as err:
                             item_errors[key] = err
                     else:
                         items[key] = value
-                if total and key_count != len(keys):
-                    for key in keys:
-                        if key not in data:
-                            item_errors[key] = MISSING_PROPERTY
+                for missing in required_keys - data.keys():
+                    item_errors[missing] = MISSING_PROPERTY
                 errors = () if constraints is None else constraints.errors(data)
                 if item_errors or errors:
                     raise ValidationError(errors, item_errors)
