@@ -3,8 +3,10 @@ from dataclasses import (  # type: ignore
     Field,
     _FIELD,
     _FIELDS,
+    _FIELD_CLASSVAR,
     _FIELD_INITVAR,
     fields as fields_,
+    is_dataclass,
 )
 from functools import wraps
 from inspect import signature
@@ -20,7 +22,6 @@ from typing import (
     overload,
 )
 
-from apischema.dataclasses import is_dataclass, is_dataclass_origin
 from apischema.utils import PREFIX
 
 FIELDS_SET_ATTR = f"{PREFIX}fields_set"
@@ -41,7 +42,7 @@ def with_fields_set(cls: Cls) -> Cls:
 
     init_fields = set()
     post_init_fields = set()
-    if is_dataclass_origin(cls):
+    if is_dataclass(cls):
         for field in getattr(cls, _FIELDS).values():
             assert isinstance(field, Field)
             if field._field_type == _FIELD_INITVAR:  # type: ignore
@@ -68,7 +69,13 @@ def with_fields_set(cls: Cls) -> Cls:
     def new_init(self, *args, **kwargs):
         prev_fields_set = self.__dict__.get(FIELDS_SET_ATTR, set()).copy()
         self.__dict__[FIELDS_SET_ATTR] = set()
-        old_init(self, *args, **kwargs)
+        try:
+            old_init(self, *args, **kwargs)
+        except TypeError as err:
+            if str(err) == no_dataclass_init_error:
+                raise RuntimeError(dataclass_before_error) from None
+            else:
+                raise
         arg_fields = {*params[: len(args)], *kwargs} - init_fields
         self.__dict__[FIELDS_SET_ATTR] = prev_fields_set | arg_fields | post_init_fields
 
@@ -76,9 +83,7 @@ def with_fields_set(cls: Cls) -> Cls:
         try:
             self.__dict__[FIELDS_SET_ATTR].add(attr)
         except KeyError:
-            raise RuntimeError(
-                f"{with_fields_set.__name__} must be put before dataclass decorator"
-            )
+            raise RuntimeError(dataclass_before_error) from None
         old_setattr(self, attr, value)
 
     for attr, old, new in [
@@ -94,9 +99,15 @@ def with_fields_set(cls: Cls) -> Cls:
     return cls
 
 
-T = TypeVar("T")
+no_dataclass_init_error = (
+    "object.__init__() takes exactly one argument (the instance to initialize)"
+)
+dataclass_before_error = (
+    f"{with_fields_set.__name__} must be put before dataclass decorator"
+)
 
-ALL_FIELDS_ATTR = f"{PREFIX}all_fields"
+
+T = TypeVar("T")
 
 
 def _all_fields(obj: Any) -> AbstractSet[str]:
@@ -151,7 +162,11 @@ def fields_set(obj: Any) -> AbstractSet[str]:
 class FieldGetter:
     def __init__(self, obj):
         _check_dataclass(obj)
-        self.fields = {f.name: f for f in fields_(obj)}
+        self.fields = {
+            name: f
+            for name, f in getattr(obj, _FIELDS).items()
+            if not f._field_type == _FIELD_CLASSVAR
+        }
 
     def __getattribute__(self, name: str) -> Field:
         try:
