@@ -223,7 +223,7 @@ class SchemaBuilder(ConversionsVisitor[Conv, Thunk[graphql.GraphQLType]]):
         return ObjectField(
             field.name,
             field_type,
-            alias=get_alias(field),
+            alias=self.aliaser(get_alias(field)),
             conversions=get_field_conversions(field, self.operation),
             default=graphql.Undefined if is_required(field) else get_default(field),
             schema=field.metadata.get(SCHEMA_METADATA),
@@ -232,9 +232,8 @@ class SchemaBuilder(ConversionsVisitor[Conv, Thunk[graphql.GraphQLType]]):
     def _visit_merged(
         self, field: Field, field_type: AnyType
     ) -> Thunk[graphql.GraphQLType]:
-        return self.visit_with_conversions(
-            field_type, get_field_conversions(field, self.operation)
-        )
+        with self._replace_conversions(get_field_conversions(field, self.operation)):
+            return self.visit(field_type)
 
     def dataclass(
         self,
@@ -264,7 +263,7 @@ class SchemaBuilder(ConversionsVisitor[Conv, Thunk[graphql.GraphQLType]]):
 
     def literal(self, values: Sequence[Any]) -> Thunk[graphql.GraphQLType]:
         if not all(isinstance(v, str) for v in values):
-            raise TypeError("Apischema GraphQL only support Enum/Literal of strings")
+            raise TypeError("apischema GraphQL only support Enum/Literal of strings")
         name, description = self._ref_and_desc
         return graphql.GraphQLEnumType(
             name, dict(zip(values, values)), description=description
@@ -307,8 +306,7 @@ class SchemaBuilder(ConversionsVisitor[Conv, Thunk[graphql.GraphQLType]]):
         raise NotImplementedError
 
     def primitive(self, cls: Type) -> Thunk[graphql.GraphQLType]:
-        if cls is NoneType:
-            raise Nullable
+        assert cls is not NoneType
         try:
             name, description = self._ref_and_desc
             return graphql.GraphQLScalarType(name, description=description)
@@ -318,17 +316,42 @@ class SchemaBuilder(ConversionsVisitor[Conv, Thunk[graphql.GraphQLType]]):
     def tuple(self, types: Sequence[AnyType]) -> Thunk[graphql.GraphQLType]:
         raise TypeError("Tuple are not supported")
 
+    def _use_cache(
+        self, key: Any, thunk: Lazy[Thunk[graphql.GraphQLType]]
+    ) -> Thunk[graphql.GraphQLType]:
+        full_key = key, self._ref, self._schema, self._conversions
+        if full_key not in self._cache:
+            cache = None
+
+            def rec_sentinel() -> graphql.GraphQLType:
+                nonlocal cache
+                assert cache is not None
+                if not isinstance(cache, graphql.GraphQLType):
+                    cache = exec_thunk(cache)
+                return cache
+
+            self._cache[full_key] = rec_sentinel
+            cache = thunk()
+
+        return self._cache[full_key]
+
     def union(self, alternatives: Sequence[AnyType]) -> Thunk[graphql.GraphQLType]:
         alternatives = list(filter_skipped(alternatives, schema_only=True))
         results = []
+        non_null_alternatives = []
         for alt in alternatives:
             try:
                 results.append(self.visit(alt))
+                non_null_alternatives.append(alt)
             except Nullable:
                 self._non_null = False
         if not results:
             raise TypeError("Empty union")
-        return self._union_result(results)
+        if len(results) == 1:
+            return results[0]
+        return self._use_cache(
+            tuple(non_null_alternatives), lambda: self._union_result(results)
+        )
 
     def visit_with_schema(
         self, tp: AnyType, ref: Optional[str], schema: Optional[Schema]
@@ -337,8 +360,6 @@ class SchemaBuilder(ConversionsVisitor[Conv, Thunk[graphql.GraphQLType]]):
             return graphql.GraphQLNonNull(self.id_type)
         if self._apply_dynamic_conversions(tp) is None:
             ref, schema = ref or get_ref(tp), merge_schema(get_schema(tp), schema)
-        else:
-            ref, schema = None, None
         ref_save, schema_save, non_null_save = self._ref, self._schema, self._non_null
         self._ref, self._schema, self._non_null = ref, schema, True
         try:
@@ -352,27 +373,20 @@ class SchemaBuilder(ConversionsVisitor[Conv, Thunk[graphql.GraphQLType]]):
             self._non_null = non_null_save
 
     def _visit(self, tp: AnyType) -> Thunk[graphql.GraphQLType]:
-        key = self._generic or tp, self._ref, self._schema, self._conversions
-        if key in self._cache:
-            return self._cache[key]
-        cache = None
-
-        def rec_sentinel() -> graphql.GraphQLType:
-            assert cache is not None
-            return cache
-
-        self._cache[key] = rec_sentinel
-        try:
-            cache = exec_thunk(super()._visit(tp))
-        except Exception:
-            del self._cache[key]
-            raise
-        else:
-            return cache
+        if tp is NoneType:
+            raise Nullable
+        _visit = super()._visit
+        return self._use_cache(self._generic or tp, lambda: _visit(tp))
 
     def visit(self, tp: AnyType) -> Thunk[graphql.GraphQLType]:
         return self.visit_with_schema(tp, None, None)
 
+    def visit_with_conversions(
+        self, tp: AnyType, conversions: Optional[Conversions]
+    ) -> Thunk[graphql.GraphQLType]:
+        with self._replace_conversions(conversions):
+            return self.visit_with_schema(tp, self._ref, self._schema)
+
 
 FieldType = TypeVar("FieldType", graphql.GraphQLInputField, graphql.GraphQLField)
 
@@ -421,7 +435,8 @@ class InputSchemaBuilder(
             except Exception:
                 field_type = Optional[field_type]
 
-        type_thunk = self.visit_with_conversions(field_type, field.conversions)
+        with self._replace_conversions(field.conversions):
+            type_thunk = self.visit(field_type)
 
         def field_thunk():
             return graphql.GraphQLInputField(
@@ -430,7 +445,7 @@ class InputSchemaBuilder(
                 description=field.description,
             )
 
-        return self.aliaser(field.alias or field.name), field_thunk
+        return field.alias or self.aliaser(field.name), field_thunk
 
     def object(
         self,
@@ -448,19 +463,30 @@ class InputSchemaBuilder(
         )
 
     def typed_dict(
-        self, cls: Type, keys: Mapping[str, AnyType], total: bool
+        self, cls: Type, keys: Mapping[str, AnyType], required_keys: Collection[str]
     ) -> Thunk[graphql.GraphQLType]:
         return self.object(
-            cls, [ObjectField(name, type) for name, type in keys.items()]
+            cls,
+            [
+                ObjectField(
+                    key,
+                    type,
+                    default=Undefined if key in required_keys else graphql.Undefined,
+                    alias=key,
+                )
+                for key, type in keys.items()
+            ],
         )
 
     def _union_result(
         self, results: Iterable[Thunk[graphql.GraphQLType]]
     ) -> Thunk[graphql.GraphQLType]:
-        results = list(results)  # Execute the iteration
+        results = list(results)
+        # Check must be done here too because _union_result is used by visit_conversion
         if len(results) == 1:
             return results[0]
-        raise TypeError("Union are not supported for input")
+        else:
+            raise TypeError("Union are not supported for input")
 
 
 class OutputSchemaBuilder(
@@ -477,7 +503,7 @@ class OutputSchemaBuilder(
         self.input_builder = InputSchemaBuilder(
             aliaser, id_type, is_id, union_ref_factory
         )
-        self._get_merged: Optional[Callable] = None
+        self._get_merged: Optional[Callable[[Any], Any]] = None
 
     def _wrap_resolve(self, resolve: Callable):
         if self._get_merged is None:
@@ -507,7 +533,8 @@ class OutputSchemaBuilder(
         field_type = field.type
         if is_union_of(field_type, UndefinedType):
             field_type = Optional[field_type]
-        type_thunk = self.visit_with_conversions(field_type, field.conversions)
+        with self._replace_conversions(field.conversions):
+            type_thunk = self.visit(field_type)
         args = None
         if field.parameters is not None:
             parameters, types, params_metadata = field.parameters
@@ -536,9 +563,8 @@ class OutputSchemaBuilder(
                 conversions = metadata.get(
                     CONVERSIONS_METADATA, ConversionMetadata()
                 ).deserialization
-                arg_type = self.input_builder.visit_with_conversions(
-                    param_type, conversions
-                )
+                with self._replace_conversions(conversions):
+                    arg_type = self.input_builder.visit(param_type)
                 description = None
                 if SCHEMA_METADATA in metadata:
                     schema: Schema = metadata[SCHEMA_METADATA]
@@ -553,7 +579,7 @@ class OutputSchemaBuilder(
                     )
 
                 args[self.aliaser(param.name)] = arg_thunk
-        return self.aliaser(field.alias or field.name), lambda: graphql.GraphQLField(
+        return field.alias or self.aliaser(field.name), lambda: graphql.GraphQLField(
             exec_thunk(type_thunk),
             {name: arg() for name, arg in args.items()} if args else None,
             resolve,
@@ -579,7 +605,8 @@ class OutputSchemaBuilder(
         merged_save = self._get_merged
         self._get_merged = get_merge
         try:
-            return self.visit_with_conversions(field_type, conversions)
+            with self._replace_conversions(conversions):
+                return self.visit(field_type)
         finally:
             self._get_merged = merged_save
 
@@ -590,24 +617,17 @@ class OutputSchemaBuilder(
         merged_types: Mapping[str, Thunk[graphql.GraphQLType]] = None,
     ) -> Thunk[graphql.GraphQLType]:
         fields_and_resolvers = list(fields)
-        try:
-            name, description = self._ref_and_desc
-        except MissingRef:
-            if cls.__name__ not in ("Query", "Mutation", "Subscription"):
-                raise
-            name, description = cls.__name__, self._description
+        name, description = self._ref_and_desc
         for resolver_alias, (resolver, types) in get_resolvers(
             self._generic or cls
         ).items():
             resolver_field = ObjectField(
                 resolver.func.__name__,
                 types["return"],
-                alias=resolver_alias,
+                alias=self.aliaser(resolver_alias),
                 conversions=resolver.conversions,
                 parameters=(resolver.parameters, types, resolver.parameters_metadata),
-                resolve=self._wrap_resolve(
-                    resolver_resolve(resolver, types, self.aliaser)
-                ),
+                resolve=resolver_resolve(resolver, types, self.aliaser),
             )
             fields_and_resolvers.append(resolver_field)
         fields_and_resolvers = sort_by_annotations_position(
@@ -654,21 +674,21 @@ class OutputSchemaBuilder(
             )
 
     def typed_dict(
-        self, cls: Type, keys: Mapping[str, AnyType], total: bool
+        self, cls: Type, keys: Mapping[str, AnyType], required_keys: Collection[str]
     ) -> Thunk[graphql.GraphQLType]:
         raise TypeError("TyedDict are not supported in output schema")
 
     def _union_result(
         self, results: Iterable[Thunk[graphql.GraphQLType]]
     ) -> Thunk[graphql.GraphQLType]:
-        results = list(results)  # Execute the iteration
-        if len(results) == 1:
-            return results[0]
+        results = list(results)  # Execute the iteration (tuple to be hashable)
         name, description = self._ref, self._description
         if name is None and self.union_ref_factory is None:
             raise MissingRef
 
         def thunk() -> graphql.GraphQLUnionType:
+            # No need to use a thunk here because union can only have class members,
+            # which use already thunks.
             types = [exec_thunk(res, non_null=False) for res in results]
             if name is None:
                 assert self.union_ref_factory is not None
@@ -866,8 +886,9 @@ def graphql_schema(
     def root_type(
         name: str, fields: Collection[ObjectField]
     ) -> Optional[graphql.GraphQLObjectType]:
-        if not fields and name != "Query":
+        if not fields:
             return None
+        builder._ref = name
         return exec_thunk(builder.object(type(name, (), {}), fields), non_null=False)
 
     return graphql.GraphQLSchema(
