@@ -66,7 +66,6 @@ from apischema.metadata.keys import (
 from apischema.serialization import serialize
 from apischema.serialization.serialized_methods import get_serialized_methods
 from apischema.skip import filter_skipped
-from apischema.tagged_unions import TAGS_ATTR
 from apischema.types import AnyType, OrderedDict
 from apischema.typing import get_origin
 from apischema.utils import UndefinedType, is_union_of, sort_by_annotations_position
@@ -283,14 +282,6 @@ class SchemaBuilder(ConversionsVisitor[Conv, JsonSchema]):
                 cls, DependentRequired.requiring, self.operation
             ).items()
         }
-        if hasattr(cls, TAGS_ATTR):
-            return json_schema(
-                type=JsonType.OBJECT,
-                oneOf=[
-                    json_schema(properties={prop: value})
-                    for prop, value in properties.items()
-                ],
-            )
         result = json_schema(
             type=JsonType.OBJECT,
             properties=properties,
@@ -402,13 +393,13 @@ class SchemaBuilder(ConversionsVisitor[Conv, JsonSchema]):
 
     @with_schema
     def typed_dict(
-        self, cls: Type, keys: Mapping[str, AnyType], total: bool
+        self, cls: Type, keys: Mapping[str, AnyType], required_keys: Collection[str]
     ) -> JsonSchema:
         self._check_constraints(ObjectConstraints)
         return json_schema(
             type=JsonType.OBJECT,
             properties={key: self.visit(tp) for key, tp in keys.items()},
-            required=list(keys) if total else [],
+            required=[key for key in keys if key in required_keys],  # preserve order
         )
 
     def _union_result(self, results: Iterable[JsonSchema]) -> JsonSchema:
